package com.xhkjedu.smarking.controller.papercheck;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.xhkjedu.smarking.service.papercheck.MsPaperCheckTeacherTaskService;
import com.xhkjedu.smarking.vo.papercheck.MsCheckParam;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.utils.PageUtil;
import com.xhkjedu.vo.PageResult;
import com.xhkjedu.vo.ResultVo;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @Description 阅卷批阅教师任务表相关操作
 * @Author auto
 * @Date 2024-10-15
 */
@RestController
@RequestMapping("/mtask")
public class MsPaperCheckTeacherTaskController {
    @Resource
    private MsPaperCheckTeacherTaskService msPaperCheckTeacherTaskService;

    /**
     * @Description 阅卷进度-按题块
     * @Date 2024/10/30 10:23
     * @Author YWX
     * @Param [param]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/list_by_block")
    public ResultVo listByBlock(@RequestBody MsCheckParam param) {
        Integer mpid = param.getMpid();
        N_Utils.validation(new Object[]{mpid, "试卷ID", 1});
        List<Map> list = msPaperCheckTeacherTaskService.listByBlock(mpid);
        return new ResultVo(0, "查询成功", list);
    }

    /**
     * @Description 阅卷进度-按题块-查看详情
     * @Date 2024/10/30 10:42
     * @Author YWX
     * @Param [param]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/detail_by_block")
    public ResultVo detailByBlock(@RequestBody MsCheckParam param) {
        Integer mpid = param.getMpid();
        Integer mblockid = param.getMblockid();
        N_Utils.validation(new Object[]{mpid, "试卷ID", 1, mblockid, "题块ID", 1});
        List<Map> list = msPaperCheckTeacherTaskService.detailByBlock(mpid, mblockid);
        return new ResultVo(0, "查询成功", list);
    }

    /**
     * @Description 阅卷进度-按教师
     * @Date 2024/10/30 10:52
     * @Author YWX
     * @Param [param]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/list_by_teacher")
    public ResultVo listByTeacher(@RequestBody MsCheckParam param) {
        Integer mpid = param.getMpid();
        N_Utils.validation(new Object[]{mpid, "试卷ID", 1});
        List<Map> list = msPaperCheckTeacherTaskService.listByTeacher(mpid);
        return new ResultVo(0, "查询成功", list);
    }

    /**
     * @Description 教师阅卷-待阅卷列表
     * @Date 2024/10/30 14:35
     * @Author YWX
     * @Param [param]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/list_pending")
    public ResultVo listPending(@RequestBody MsCheckParam param) {
        Integer page = param.getPage();
        Integer size = param.getPageSize();
        N_Utils.validation(new Object[]{page, "显示页码", 1, size, "显示条数", 1});
        msPaperCheckTeacherTaskService.setParam(param);
        PageHelper.startPage(page, size);
        N_Utils.validation(new Object[]{param.getHandleid(), "操作人ID", 1});
        List<Map> list = msPaperCheckTeacherTaskService.listPending(param);
        PageResult pageResult = PageUtil.getPageResult(new PageInfo<>(list));
        return new ResultVo(0, "查询成功", pageResult);
    }

    /**
     * @Description 教师阅卷-待阅卷详情
     * @Date 2024/10/30 16:02
     * @Author YWX
     * @Param [param]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/list_pending_block")
    public ResultVo listPendingBlock(@RequestBody MsCheckParam param) {
        N_Utils.validation(new Object[]{param.getHandleid(), "操作人ID", 1, param.getMpid(), "试卷ID", 1});
        List<Map> list = msPaperCheckTeacherTaskService.listPendingBlock(param);
        return new ResultVo(0, "查询成功", list);
    }

    /**
     * @Description 教师阅卷-题块列表
     * @Date 2024/10/31 9:40
     * @Author YWX
     * @Param [param]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/list_teacher_block")
    public ResultVo listTeacherBlock(@RequestBody MsCheckParam param) {
        N_Utils.validation(new Object[]{param.getHandleid(), "操作人ID", 1, param.getMpid(), "试卷ID", 1});
        List<Map> list = msPaperCheckTeacherTaskService.listTeacherBlock(param);
        return new ResultVo(0, "查询成功", list);
    }

    /**
     * @Description 获取待批题块
     * @Date 2024/10/31 11:02
     * @Author YWX
     * @Param [param]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/get_correct_block")
    public ResultVo getCorrectBlock(@RequestBody MsCheckParam param) {
        N_Utils.validation(new Object[]{param.getHandleid(), "操作人ID", 1, param.getMpid(), "试卷ID", 1});
        return new ResultVo(0, "查询成功", msPaperCheckTeacherTaskService.getCorrectBlock(param));
    }

    /**
     * @Description 阅卷进度-按试题
     * @Date 2024/11/8 16:19
     * @Author YWX
     * @Param [param]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/list_by_question")
    public ResultVo listByQuestion(@RequestBody MsCheckParam param) {
        Integer mpid = param.getMpid();
        N_Utils.validation(new Object[]{mpid, "试卷ID", 1});
        List<Map> list = msPaperCheckTeacherTaskService.listByQuestion(mpid);
        return new ResultVo(0, "查询成功", list);
    }
}
