package com.xhkjedu.smarking.service.papercheck;

import com.xhkjedu.exception.ServiceException;
import com.xhkjedu.smarking.mapper.paper.MsPaperMapper;
import com.xhkjedu.smarking.mapper.papercheck.MsPaperCheckMapper;
import com.xhkjedu.smarking.mapper.papercheck.MsPaperCheckTeacherTaskMapper;
import com.xhkjedu.smarking.mapper.system.UserMapper;
import com.xhkjedu.smarking.model.paper.MsPaper;
import com.xhkjedu.smarking.model.papercheck.MsPaperCheck;
import com.xhkjedu.smarking.vo.papercheck.MsCheckParam;
import com.xhkjedu.smarking.vo.papercheck.MsPaperCheckVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import com.xhkjedu.vo.system.UserVo;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * @Description 阅卷批阅教师任务表 服务实现类
 * @Author auto
 * @Date 2024-10-15
 */
@Service
public class MsPaperCheckTeacherTaskService {
    @Resource
    private MsPaperCheckTeacherTaskMapper msPaperCheckTeacherTaskMapper;
    @Resource
    private UserMapper userMapper;
    @Resource
    private MsPaperCheckMapper msPaperCheckMapper;
    @Resource
    private MsPaperMapper msPaperMapper;

    /**
     * @Description 阅卷进度-按题块
     * @Date 2024/10/30 10:23
     * @Author YWX
     * @Param [mpid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listByBlock(Integer mpid) {
        return msPaperCheckTeacherTaskMapper.listByBlock(mpid);
    }

    /**
     * @Description 阅卷进度-按题块-查看详情
     * @Date 2024/10/30 10:44
     * @Author YWX
     * @Param [mblockid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> detailByBlock(Integer mpid, Integer mblockid) {
        MsPaperCheck check = msPaperCheckMapper.findByMBlockId(mpid, mblockid);
        if (check == null) {
            throw new ServiceException("该题块不存在");
        }
        Integer dispenseway = check.getDispenseway();
        List<Map> list = msPaperCheckTeacherTaskMapper.detailByBlock(mpid, mblockid, dispenseway, check.getChecktype());
        if (dispenseway.equals(2)) {
            int totalNum = list.stream().mapToInt(m -> N_Utils.obj2Int(m.get("totalNum"))).sum();
            int totalCheckedNum = list.stream().mapToInt(m -> N_Utils.obj2Int(m.get("checkedNum"))).sum();
            int uncheckedNum = totalNum - totalCheckedNum;
            for (Map m : list) {
                m.put("totalNum", totalNum);
                m.put("uncheckedNum", uncheckedNum);
            }
        }
        return list;
    }

    /**
     * @Description 阅卷进度-按教师
     * @Date 2024/10/30 10:52
     * @Author YWX
     * @Param [mpid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listByTeacher(Integer mpid) {
        return msPaperCheckTeacherTaskMapper.listByTeacher(mpid);
    }

    /**
     * @Description 教师阅卷-待阅卷列表
     * @Date 2024/10/30 14:41
     * @Author YWX
     * @Param [handleid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listPending(MsCheckParam param) {
        return msPaperCheckTeacherTaskMapper.listPending(param);
    }

    //设置查询参数
    public void setParam(MsCheckParam param) {
        UserVo vo = userMapper.getUserInfoForHandleLog(param.getHandleid());
        param.setSchoolid(vo.getSchoolid());
        param.setUsertype(vo.getUsertype());
    }

    /**
     * @Description 教师阅卷-待阅卷详情
     * @Date 2024/10/30 16:02
     * @Author YWX
     * @Param [param]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listPendingBlock(MsCheckParam param) {
        setParam(param);
        return msPaperCheckTeacherTaskMapper.listPendingBlock(param);
    }

    /**
     * @Description 教师阅卷-题块列表
     * @Date 2024/10/31 9:49
     * @Author YWX
     * @Param [param]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listTeacherBlock(MsCheckParam param) {
        setParam(param);
        return msPaperCheckTeacherTaskMapper.listTeacherBlock(param);
    }

    /**
     * @Description 获取待批题块
     * @Date 2024/10/31 11:01
     * @Author YWX
     * @Param [param]
     * @Return java.util.Map
     **/
    public ResultVo getCorrectBlock(MsCheckParam param) {
        setParam(param);

        List<MsPaperCheck> checks = msPaperCheckTeacherTaskMapper.listPaperCheck(param);
        if (N_Utils.isListEmpty(checks)) throw new ServiceException("暂无要批阅的题块");
        MsPaperCheckVo sq = null;
        for (MsPaperCheck check : checks) {
            Integer checktype = check.getChecktype();
            param.setChecktype(checktype);
            Integer dispenseway = check.getDispenseway();
            if (dispenseway.equals(2)) {//效率优先
                param.setChecktype(check.getChecktype());
                sq = msPaperCheckTeacherTaskMapper.getCorrectBlock2(param);
            } else {
                sq = msPaperCheckTeacherTaskMapper.getCorrectBlock(param);
            }
            if (sq != null) {
                break;
            }
        }
        Map map = new HashMap();
        map.put("sq", sq);
        String msg = "获取成功";
        return new ResultVo(0, msg, map);
    }

    /**
     * @Description 阅卷进度-按试题
     * @Date 2024/11/8 16:18
     * @Author YWX
     * @Param [mpid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listByQuestion(Integer mpid) {
        MsPaper paper = msPaperMapper.selectByPrimaryKey(mpid);
        if (paper == null) {
            throw new ServiceException("该试卷不存在");
        }
        Integer correcttype = paper.getCorrecttype();
        if (!N_Utils.isTrueInteger(correcttype)) {
            throw new ServiceException("该试卷未设置批阅方式");
        }
        return msPaperCheckTeacherTaskMapper.listByQuestion(mpid, correcttype);
    }
}
